@@ -1,16 +1,17 @@
 import abc
-from typing import Any, List, Dict
+from typing import Any, Dict, List
 
 import pandas as pd
 
+
 class DynamicTransformBase(abc.ABC):
-    """Abstract class for a dynamic transform as seen by hamilton.
+    """Abstract class for a dynamic transform transform as seen by hamilton.
     These are transforms that come from configuration parameters, and define the following:
     1. The nodes that they depend on
     2. What the transform does
 
     Paired with the decorator @dynamic_transform(CLS, config_item, **extra_params) one can write incredibly powerful
-    DAGs that depend on dynamic configs.
+    DAGs that depend on dynamic transform configs.
     """
 
     def __init__(self, config_parameters: Any, name: str):
@@ -55,11 +56,12 @@ class BaseModel(DynamicTransformBase, abc.ABC):
         #column that is some combinations of other columns using some previously trained model
 
     Note that the model-training is not yet in scope."""
+
     def compute(self, **inputs: Any) -> Any:
         """Delegates to predict function to compute this model's return.
 
         :param inputs: Inputs for the model
-        :return: The result of claling the model.
+        :return: The result of calling the model.
         """
         return self.predict(**inputs)
 
