@@ -1,9 +1,10 @@
 import collections
 import random
+from typing import Tuple
 
 import tests.resources.reuse_subdag
 from hamilton import ad_hoc_utils, graph
-from hamilton.function_modifiers import config, recursive, value
+from hamilton.function_modifiers import config, parameterized_subdag, recursive, subdag, value
 from hamilton.function_modifiers.dependencies import source
 
 
@@ -138,6 +139,20 @@ def test_reuse_subdag_allows_config_as_input():
     assert nodes["baz.bar"](**{"baz.b": 100}) == 100
 
 
+def test_subdag_empty_config():
+    @config.when_not(some_config_param=False)
+    def baz(foo: int, bar: int) -> int:
+        return foo + bar
+
+    def subdag(baz: int) -> int:
+        return baz
+
+    decorator = recursive.subdag(baz, inputs={})
+    nodes = {node_.name: node_ for node_ in decorator.generate_nodes(subdag, {})}
+    assert "subdag.baz" in nodes
+    assert "subdag" in nodes
+
+
 def test_reuse_subdag_end_to_end():
     fg = graph.FunctionGraph(tests.resources.reuse_subdag, config={"op": "subtract"})
     prefixless_nodes = []
@@ -209,3 +224,86 @@ def test_reuse_subdag_end_to_end():
     assert fg.nodes["v3.d"].callable(**{"v3.c": 10, "a": 100}) == 100 + 10
     res = fg.execute(nodes=[fg.nodes["sum_everything"]])
     assert res["sum_everything"] == 318
+
+
+def test_parameterize_subdag():
+    def bar(input_1: int) -> int:
+        return input_1 + 1
+
+    def foo(input_2: int) -> int:
+        return input_2 + 1
+
+    @config.when(baz_version="standard")
+    def baz__standard(foo: int, bar: int) -> int:
+        return foo + bar
+
+    @config.when(baz_version="alternate")
+    def baz__alternate(foo: int, bar: int) -> int:
+        return foo * bar
+
+    def subdag_processor(foo: int, bar: int, baz: int) -> Tuple[int, int, int]:
+        return foo, bar, baz
+
+    fns = [bar, foo, baz__standard, baz__alternate]
+
+    decorator = parameterized_subdag(
+        *fns,
+        inputs={"input_1": source("external_input_1"), "input_2": source("external_input_2")},
+        config={"baz_version": "standard"},
+        v0={},
+        v1={"config": {"baz_version": "standard"}, "inputs": {"input_1": value(100)}},
+        v2={
+            "config": {"baz_version": "alternate"},
+            "inputs": {"input_2": value(200)},
+        },
+    )
+    dag_generated = decorator.generate_nodes(subdag_processor, {})
+    assert len(dag_generated) == len(set([item.name for item in dag_generated]))
+    nodes_by_name = {node.name: node for node in dag_generated}
+    assert "v0" in nodes_by_name
+    assert "v1" in nodes_by_name
+    assert "v2" in nodes_by_name
+    assert "v0.baz" in nodes_by_name
+    assert "v1.baz" in nodes_by_name
+    assert "v2.baz" in nodes_by_name
+    assert nodes_by_name["v0.baz"].callable(**{"v0.foo": 1, "v0.bar": 2}) == 3
+    assert nodes_by_name["v1.baz"].callable(**{"v1.foo": 1, "v1.bar": 2}) == 3
+    assert nodes_by_name["v2.baz"].callable(**{"v2.foo": 1, "v2.bar": 2}) == 2
+
+
+def test_nested_subdag():
+    def bar(input_1: int) -> int:
+        return input_1 + 1
+
+    def foo(input_2: int) -> int:
+        return input_2 + 1
+
+    @subdag(
+        foo,
+        bar,
+    )
+    def inner_subdag(foo: int, bar: int) -> Tuple[int, int]:
+        return foo, bar
+
+    @subdag(inner_subdag, inputs={"input_2": value(10)}, config={"plus_one": True})
+    def outer_subdag_1(inner_subdag: Tuple[int, int]) -> int:
+        return sum(inner_subdag)
+
+    @subdag(inner_subdag, inputs={"input_2": value(3)}, config={"plus_one": False})
+    def outer_subdag_2(inner_subdag: Tuple[int, int]) -> int:
+        return sum(inner_subdag)
+
+    def sum_all(outer_subdag_1: int, outer_subdag_2: int) -> int:
+        return outer_subdag_1 + outer_subdag_2
+
+    # we only need to generate from the outer subdag
+    # as it refers to the inner one
+    full_module = ad_hoc_utils.create_temporary_module(outer_subdag_1, outer_subdag_2, sum_all)
+    fg = graph.FunctionGraph(full_module, config={})
+    assert "outer_subdag_1" in fg.nodes
+    assert "outer_subdag_2" in fg.nodes
+    res = fg.execute(nodes=[fg.nodes["sum_all"]], inputs={"input_1": 2})
+    # This is effectively the function graph
+    assert res["sum_all"] == sum_all(
+        outer_subdag_1(inner_subdag(bar(2), foo(10))), outer_subdag_2(inner_subdag(bar(2), foo(3)))
+    )
